@@ -19,17 +19,18 @@ const MAX_PATH: usize = 266;
 static EXIT_ERR: i32 = 1;
 
 #[cfg(any(
-    windows,
     target_os = "freebsd",
     target_vendor = "apple",
     target_os = "netbsd",
     target_os = "openbsd"
 ))]
+use crate::os_str_from_bytes;
 #[cfg(windows)]
 use crate::show_warning;
 
-#[cfg(windows)]
 use std::ffi::OsStr;
+#[cfg(unix)]
+use std::os::unix::ffi::OsStrExt;
 #[cfg(windows)]
 use std::os::windows::ffi::OsStrExt;
 #[cfg(windows)]
@@ -61,17 +62,23 @@ fn to_nul_terminated_wide_string(s: impl AsRef<OsStr>) -> Vec<u16> {
 use libc::{
     S_IFBLK, S_IFCHR, S_IFDIR, S_IFIFO, S_IFLNK, S_IFMT, S_IFREG, S_IFSOCK, mode_t, strerror,
 };
-use std::borrow::Cow;
 #[cfg(unix)]
-use std::ffi::CStr;
-#[cfg(unix)]
-use std::ffi::CString;
+use std::ffi::{CStr, CString};
 use std::io::Error as IOError;
 #[cfg(unix)]
 use std::mem;
 #[cfg(windows)]
 use std::path::Path;
+use std::time::SystemTime;
+#[cfg(not(windows))]
 use std::time::UNIX_EPOCH;
+use std::{borrow::Cow, ffi::OsString};
+
+use std::fs::Metadata;
+#[cfg(unix)]
+use std::os::unix::fs::MetadataExt;
+#[cfg(unix)]
+use std::time::Duration;
 
 #[cfg(any(
     target_os = "linux",
@@ -109,28 +116,73 @@ pub use libc::statfs as statfs_fn;
 ))]
 pub use libc::statvfs as statfs_fn;
 
-pub trait BirthTime {
-    fn birth(&self) -> Option<(u64, u32)>;
+#[derive(Debug, Copy, Clone)]
+pub enum MetadataTimeField {
+    Modification,
+    Access,
+    Change,
+    Birth,
 }
 
-use std::fs::Metadata;
-impl BirthTime for Metadata {
-    fn birth(&self) -> Option<(u64, u32)> {
-        self.created()
-            .ok()
-            .and_then(|t| t.duration_since(UNIX_EPOCH).ok())
-            .map(|e| (e.as_secs(), e.subsec_nanos()))
+impl From<&str> for MetadataTimeField {
+    /// Get a `MetadataTimeField` from a string, we expect the value
+    /// to come from clap, and be constrained there (e.g. if Modification is
+    /// not supported), and the default branch should not be reached.
+    fn from(value: &str) -> Self {
+        match value {
+            "ctime" | "status" => Self::Change,
+            "access" | "atime" | "use" => Self::Access,
+            "mtime" | "modification" => Self::Modification,
+            "birth" | "creation" => Self::Birth,
+            // below should never happen as clap already restricts the values.
+            _ => unreachable!("Invalid metadata time field."),
+        }
     }
 }
 
+#[cfg(unix)]
+fn metadata_get_change_time(md: &Metadata) -> Option<SystemTime> {
+    let mut st = UNIX_EPOCH;
+    let (secs, nsecs) = (md.ctime(), md.ctime_nsec());
+    if secs >= 0 {
+        st += Duration::from_secs(secs as u64);
+    } else {
+        st -= Duration::from_secs(-secs as u64);
+    }
+    if nsecs >= 0 {
+        st += Duration::from_nanos(nsecs as u64);
+    } else {
+        // Probably never the case, but cover just in case.
+        st -= Duration::from_nanos(-nsecs as u64);
+    }
+    Some(st)
+}
+
+#[cfg(not(unix))]
+fn metadata_get_change_time(_md: &Metadata) -> Option<SystemTime> {
+    // Not available.
+    None
+}
+
+pub fn metadata_get_time(md: &Metadata, md_time: MetadataTimeField) -> Option<SystemTime> {
+    match md_time {
+        MetadataTimeField::Change => metadata_get_change_time(md),
+        MetadataTimeField::Modification => md.modified().ok(),
+        MetadataTimeField::Access => md.accessed().ok(),
+        MetadataTimeField::Birth => md.created().ok(),
+    }
+}
+
+// TODO: Types for this struct are probably mostly wrong. Possibly, most of them
+// should be OsString.
 #[derive(Debug, Clone)]
 pub struct MountInfo {
     /// Stores `volume_name` in windows platform and `dev_id` in unix platform
     pub dev_id: String,
     pub dev_name: String,
     pub fs_type: String,
-    pub mount_root: String,
-    pub mount_dir: String,
+    pub mount_root: OsString,
+    pub mount_dir: OsString,
     /// We only care whether this field contains "bind"
     pub mount_option: String,
     pub remote: bool,
@@ -138,7 +190,9 @@ pub struct MountInfo {
 }
 
 #[cfg(any(target_os = "linux", target_os = "android"))]
-fn replace_special_chars(s: String) -> String {
+fn replace_special_chars(s: &[u8]) -> Vec<u8> {
+    use bstr::ByteSlice;
+
     // Replace
     //
     // * ASCII space with a regular space character,
@@ -152,7 +206,11 @@ fn replace_special_chars(s: String) -> String {
 
 impl MountInfo {
     #[cfg(any(target_os = "linux", target_os = "android"))]
-    fn new(file_name: &str, raw: &[&str]) -> Option<Self> {
+    fn new(file_name: &str, raw: &[&[u8]]) -> Option<Self> {
+        use std::ffi::OsStr;
+        use std::os::unix::ffi::OsStrExt;
+        use std::os::unix::ffi::OsStringExt;
+
         let dev_name;
         let fs_type;
         let mount_root;
@@ -165,24 +223,27 @@ impl MountInfo {
             // "man proc" for more details
             LINUX_MOUNTINFO => {
                 const FIELDS_OFFSET: usize = 6;
-                let after_fields = raw[FIELDS_OFFSET..].iter().position(|c| *c == "-").unwrap()
+                let after_fields = raw[FIELDS_OFFSET..]
+                    .iter()
+                    .position(|c| *c == b"-")
+                    .unwrap()
                     + FIELDS_OFFSET
                     + 1;
-                dev_name = raw[after_fields + 1].to_string();
-                fs_type = raw[after_fields].to_string();
-                mount_root = raw[3].to_string();
-                mount_dir = replace_special_chars(raw[4].to_string());
-                mount_option = raw[5].to_string();
+                dev_name = String::from_utf8_lossy(raw[after_fields + 1]).to_string();
+                fs_type = String::from_utf8_lossy(raw[after_fields]).to_string();
+                mount_root = OsStr::from_bytes(raw[3]).to_owned();
+                mount_dir = OsString::from_vec(replace_special_chars(raw[4]));
+                mount_option = String::from_utf8_lossy(raw[5]).to_string();
             }
             LINUX_MTAB => {
-                dev_name = raw[0].to_string();
-                fs_type = raw[2].to_string();
-                mount_root = String::new();
-                mount_dir = replace_special_chars(raw[1].to_string());
-                mount_option = raw[3].to_string();
+                dev_name = String::from_utf8_lossy(raw[0]).to_string();
+                fs_type = String::from_utf8_lossy(raw[2]).to_string();
+                mount_root = OsString::new();
+                mount_dir = OsString::from_vec(replace_special_chars(raw[1]));
+                mount_option = String::from_utf8_lossy(raw[3]).to_string();
             }
             _ => return None,
-        };
+        }
 
         let dev_id = mount_dev_id(&mount_dir);
         let dummy = is_dummy_filesystem(&fs_type, &mount_option);
@@ -233,6 +294,8 @@ impl MountInfo {
             // TODO: support the case when `GetLastError()` returns `ERROR_MORE_DATA`
             return None;
         }
+        // TODO: This should probably call `OsString::from_wide`, but unclear if
+        // terminating zeros need to be striped first.
         let mount_root = LPWSTR2String(&mount_root_buf);
 
         let mut fs_type_buf = [0u16; MAX_PATH];
@@ -263,8 +326,8 @@ impl MountInfo {
             dev_id: volume_name,
             dev_name,
             fs_type: fs_type.unwrap_or_default(),
-            mount_root,
-            mount_dir: String::new(),
+            mount_root: mount_root.into(), // TODO: We should figure out how to keep an OsString here.
+            mount_dir: OsString::new(),
             mount_option: String::new(),
             remote,
             dummy: false,
@@ -292,12 +355,11 @@ impl From<StatFs> for MountInfo {
                 .to_string_lossy()
                 .into_owned()
         };
-        let mount_dir = unsafe {
+        let mount_dir_bytes = unsafe {
             // spell-checker:disable-next-line
-            CStr::from_ptr(&statfs.f_mntonname[0])
-                .to_string_lossy()
-                .into_owned()
+            CStr::from_ptr(&statfs.f_mntonname[0]).to_bytes()
         };
+        let mount_dir = os_str_from_bytes(mount_dir_bytes).unwrap().into_owned();
 
         let dev_id = mount_dev_id(&mount_dir);
         let dummy = is_dummy_filesystem(&fs_type, "");
@@ -308,7 +370,7 @@ impl From<StatFs> for MountInfo {
             dev_name,
             fs_type,
             mount_dir,
-            mount_root: String::new(),
+            mount_root: OsString::new(),
             mount_option: String::new(),
             remote,
             dummy,
@@ -343,7 +405,7 @@ fn is_remote_filesystem(dev_name: &str, fs_type: &str) -> bool {
 }
 
 #[cfg(all(unix, not(any(target_os = "aix", target_os = "redox"))))]
-fn mount_dev_id(mount_dir: &str) -> String {
+fn mount_dev_id(mount_dir: &OsStr) -> String {
     use std::os::unix::fs::MetadataExt;
 
     if let Ok(stat) = std::fs::metadata(mount_dir) {
@@ -426,10 +488,10 @@ pub fn read_fs_list() -> UResult<Vec<MountInfo>> {
             .or_else(|_| File::open(LINUX_MTAB).map(|f| (LINUX_MTAB, f)))?;
         let reader = BufReader::new(f);
         Ok(reader
-            .lines()
+            .split(b'\n')
             .map_while(Result::ok)
             .filter_map(|line| {
-                let raw_data = line.split_whitespace().collect::<Vec<&str>>();
+                let raw_data = line.split(|c| *c == b' ').collect::<Vec<&[u8]>>();
                 MountInfo::new(file_name, &raw_data)
             })
             .collect::<Vec<_>>())
@@ -814,7 +876,7 @@ impl FsMeta for StatFs {
     fn fsid(&self) -> u64 {
         // Use type inference to determine the type of f_fsid
         // (libc::__fsid_t on Android, libc::fsid_t on other platforms)
-        let f_fsid: &[u32; 2] = unsafe { &*(&self.f_fsid as *const _ as *const [u32; 2]) };
+        let f_fsid: &[u32; 2] = unsafe { &*(&raw const self.f_fsid as *const [u32; 2]) };
         ((u64::from(f_fsid[0])) << 32) | u64::from(f_fsid[1])
     }
     #[cfg(not(any(
@@ -855,11 +917,13 @@ impl FsMeta for StatFs {
 }
 
 #[cfg(unix)]
-pub fn statfs<P>(path: P) -> Result<StatFs, String>
-where
-    P: Into<Vec<u8>>,
-{
-    match CString::new(path) {
+pub fn statfs(path: &OsStr) -> Result<StatFs, String> {
+    #[cfg(unix)]
+    let p = path.as_bytes();
+    #[cfg(not(unix))]
+    let p = path.into_string().unwrap();
+
+    match CString::new(p) {
         Ok(p) => {
             let mut buffer: StatFs = unsafe { mem::zeroed() };
             unsafe {
@@ -921,6 +985,7 @@ pub fn pretty_fstype<'a>(fstype: i64) -> Cow<'a, str> {
         0x7372_7279 => "btrfs_test".into(),
         0x00C3_6400 => "ceph".into(),
         0x0027_E0EB => "cgroupfs".into(),
+        0x6367_7270 => "cgroup2fs".into(),
         0xFF53_4D42 => "cifs".into(),
         0x7375_7245 => "coda".into(),
         0x012F_F7B7 => "coh".into(),
@@ -1060,8 +1125,8 @@ mod tests {
         // spell-checker:ignore (word) relatime
         let info = MountInfo::new(
             LINUX_MOUNTINFO,
-            &"106 109 253:6 / /mnt rw,relatime - xfs /dev/fs0 rw"
-                .split_ascii_whitespace()
+            &b"106 109 253:6 / /mnt rw,relatime - xfs /dev/fs0 rw"
+                .split(|c| *c == b' ')
                 .collect::<Vec<_>>(),
         )
         .unwrap();
@@ -1075,8 +1140,8 @@ mod tests {
         // Test parsing with different amounts of optional fields.
         let info = MountInfo::new(
             LINUX_MOUNTINFO,
-            &"106 109 253:6 / /mnt rw,relatime master:1 - xfs /dev/fs0 rw"
-                .split_ascii_whitespace()
+            &b"106 109 253:6 / /mnt rw,relatime master:1 - xfs /dev/fs0 rw"
+                .split(|c| *c == b' ')
                 .collect::<Vec<_>>(),
         )
         .unwrap();
@@ -1086,8 +1151,8 @@ mod tests {
 
         let info = MountInfo::new(
             LINUX_MOUNTINFO,
-            &"106 109 253:6 / /mnt rw,relatime master:1 shared:2 - xfs /dev/fs0 rw"
-                .split_ascii_whitespace()
+            &b"106 109 253:6 / /mnt rw,relatime master:1 shared:2 - xfs /dev/fs0 rw"
+                .split(|c| *c == b' ')
                 .collect::<Vec<_>>(),
         )
         .unwrap();
@@ -1101,8 +1166,8 @@ mod tests {
     fn test_mountinfo_dir_special_chars() {
         let info = MountInfo::new(
             LINUX_MOUNTINFO,
-            &r#"317 61 7:0 / /mnt/f\134\040\011oo rw,relatime shared:641 - ext4 /dev/loop0 rw"#
-                .split_ascii_whitespace()
+            &br#"317 61 7:0 / /mnt/f\134\040\011oo rw,relatime shared:641 - ext4 /dev/loop0 rw"#
+                .split(|c| *c == b' ')
                 .collect::<Vec<_>>(),
         )
         .unwrap();
@@ -1111,12 +1176,43 @@ mod tests {
 
         let info = MountInfo::new(
             LINUX_MTAB,
-            &r#"/dev/loop0 /mnt/f\134\040\011oo ext4 rw,relatime 0 0"#
-                .split_ascii_whitespace()
+            &br#"/dev/loop0 /mnt/f\134\040\011oo ext4 rw,relatime 0 0"#
+                .split(|c| *c == b' ')
                 .collect::<Vec<_>>(),
         )
         .unwrap();
 
         assert_eq!(info.mount_dir, r#"/mnt/f\ 	oo"#);
     }
+
+    #[test]
+    #[cfg(any(target_os = "linux", target_os = "android"))]
+    fn test_mountinfo_dir_non_unicode() {
+        let info = MountInfo::new(
+            LINUX_MOUNTINFO,
+            &b"317 61 7:0 / /mnt/some-\xc0-dir-\xf3 rw,relatime shared:641 - ext4 /dev/loop0 rw"
+                .split(|c| *c == b' ')
+                .collect::<Vec<_>>(),
+        )
+        .unwrap();
+
+        assert_eq!(
+            info.mount_dir,
+            crate::os_str_from_bytes(b"/mnt/some-\xc0-dir-\xf3").unwrap()
+        );
+
+        let info = MountInfo::new(
+            LINUX_MOUNTINFO,
+            &b"317 61 7:0 / /mnt/some-\\040-dir-\xf3 rw,relatime shared:641 - ext4 /dev/loop0 rw"
+                .split(|c| *c == b' ')
+                .collect::<Vec<_>>(),
+        )
+        .unwrap();
+
+        // Note that the \040 above will have been substituted by a space.
+        assert_eq!(
+            info.mount_dir,
+            crate::os_str_from_bytes(b"/mnt/some- -dir-\xf3").unwrap()
+        );
+    }
 }
